@@ -10,9 +10,11 @@ import math
 import sys
 import traceback
 import numpy as np
+import shapefile
 from . import vtk
 from . import geojson
 from . import xyfile
+from . import _shpfuncs
 from .. import crs
 
 from collections import deque
@@ -120,6 +122,12 @@ class Point(Geometry):
             self.rank = 2
         return
 
+    def isat(self, other, tol=0.001):
+        """ Test whether point vertices are the same, regardless of properties or data.
+        Tolarance is not geodetic. """
+        return False not in [abs(a-b) <= tol for (a, b) in zip(self.vertices,
+                                                               other.vertices)]
+
     def get_vertex(self):
         """ Return the Point vertex as a tuple. """
         return self.vertex
@@ -409,7 +417,7 @@ class Multipoint(Geometry):
         elif self.rank == 3:
             f = lambda pt: (pt[0] + shift_vector[0], pt[1] + shift_vector[1],
                             pt[2] + shift_vector[2])
-        self.vertices = map(f, self.vertices)
+        self.vertices = list(map(f, self.vertices))
         return self
 
     def _matmult(self, A, x):
@@ -551,6 +559,16 @@ class Multipoint(Geometry):
         vtk.mp2vtp(self, f, **kwargs)
         return
 
+    def to_shapefile(self, fstem):
+        """ Save line to a shapefile """
+        if self.rank == 2:
+            _shpfuncs.write_multipoint2(self, fstem)
+        elif self.rank == 3:
+            _shpfuncs.write_multipoint3(self, fstem)
+        else:
+            raise IOError("rank must be 2 or 3 to write as a shapefile\n")
+        return
+
 
 class ConnectedMultipoint(Multipoint):
     """ Class for Multipoints in which vertices are assumed to be connected. """
@@ -686,6 +704,15 @@ class Line(ConnectedMultipoint):
         except NameError:
             raise GuppyError('Shapely module not available\n')
 
+    def to_shapefile(self, fstem):
+        """ Save line to a shapefile """
+        if self.rank == 2:
+            _shpfuncs.write_line2(self, fstem)
+        elif self.rank == 3:
+            _shpfuncs.write_line3(self, fstem)
+        else:
+            raise IOError("rank must be 2 or 3 to write as a shapefile\n")
+        return
 
 class Polygon(ConnectedMultipoint):
     """ This defines the polygon class, from which geographic
@@ -760,6 +787,16 @@ class Polygon(ConnectedMultipoint):
             raise ImportError('Shapely module did not import\n')
         return shp
 
+    def to_shapefile(self, fstem):
+        """ Save line to a shapefile """
+        if self.rank == 2:
+            _shpfuncs.write_poly2(self, fstem)
+        elif self.rank == 3:
+            _shpfuncs.write_poly3(self, fstem)
+        else:
+            raise IOError("rank must be 2 or 3 to write as a shapefile\n")
+        return
+
 
 class GuppyError(Exception):
     """ Base class for guppy module errors. """
